﻿// .NET Framework 4.8 による
// 中央精機製ステージコントローラ QT-A / QT-B と通信するためのサンプルです。
// サンプルのため最低限の記述を心がけています。
// エラー処理、実装方法など開発言語についてはお使いの開発環境のヘルプなどをご参照いただくか、開発言語メーカへお問い合わせください。
// 
// 【動作確認環境】
// ・ OS: Windows 10 22H2 / Windows 11 23H2
// ・ Visual C# 2022
// 
// 【前提条件】
// ・ このサンプルでは 2 軸のコントローラを動作させます。
// ・ コントローラとステージが正しく接続され、正常に動作する設定になっていることとします。
// ・ コントローラはレスポンスなし (X:0) を想定しています。
//
// 【サンプルコードの内容】
// 1. System.IO.Ports.SerialPort クラスのインスタンスを作成する
// 2. 通信ポートの設定を行う
// 3. 通信ポートを開く
// 4. QT-A / QT-B との RS-232C 通信を行う
//    - A 軸に接続されたステージを +1000 パルス移動開始します。
//    - 100 ms 待ちます。
//    - Q コマンドを送信し、返答された内容をコンソールに出力します。
// 5. 通信ポートを閉じる
//    ※ System.IO.Ports.SerialPort クラスのインスタンスを作成の際に using ステートメントを使用している場合、
//    ※ using の終了時に作成されたインスタンスは自動的に破棄され、その際に通信ポートは自動的に閉じられます。
//
// 【使用方法】
// 通信ポートの設定に合わせ、クラス内のフィールドを設定します。
// 「設定箇所 - ここから」～「設定箇所 - ここまで」の間をコントローラに合わせ変更します。

using System;
using System.IO.Ports;
using System.Linq;
using System.Threading.Tasks;

namespace CsNetFramework48Sample {
    internal class Program {
        // <<<<< 設定箇所 - ここから >>>>>
        // 通信ポートの番号を指定します。
        // 通信ポート番号はデバイスマネージャなどで確認します。
        private const string s_portName = "COM3";

        // 通信ポートのボーレート 4800, 9600, 19200, (QT-B のみ) 38400 の何れかを指定します。
        // お使いのコントローラの設定と併せてください。
        private const int s_baudrate = 9600;
        //private const int s_baudrate = 4800;
        //private const int s_baudrate = 19200;
        //private const int s_baudrate = 38400; // QT-B のみ

        // 通信ポートのデータビット長 8 か 7 bit を指定する。
        // お使いのコントローラの設定と併せてください。
        private const int s_dataBits = 8;
        //private const int s_dataBits = 7;

        // 通信ポートのパリティ Parity.None (パリティなし),　Parity.Odd (奇数パリティ), Parity.Even (偶数パリティ) の何れかを指定する。
        // お使いのコントローラの設定と併せてください。
        private const Parity s_parity = Parity.None;
        //private const Parity s_parity = Parity.Odd;
        //private const Parity s_parity = Parity.Even;

        // 通信ポートのストップビット長 StopBits.One, StopBits.Two の何れかを指定する。
        // お使いのコントローラの設定と併せてください。
        private const StopBits s_stopBits = StopBits.One;
        //private const StopBits s_stopBits = StopBits.Two;

        // コントローラとの通信で使用するデリミタを設定します。
        // お使いのコントローラの設定と併せてください。
        // \r\n: CR+LF、\r: CR、\n: LF
        private const string s_delimiter = "\r\n";
        //private const string s_delimiter = "\r";
        //private const string s_delimiter = "\n";

        // <<<<< 設定箇所 - ここまで >>>>>

        static void Main(string[] args) {
            _ = args; // コマンドライン引数を破棄

            // シリアルポートクラスのインスタンスを作成します。
            // using ステートメントから抜ける箇所で自動的に破棄 (Dispose) されます。この時通信ポートが閉じられます。
            // using ステートメントを利用しない状況では明示的に Dispose を呼び出す必要があります。
            // using ステートメント: https://learn.microsoft.com/ja-jp/dotnet/csharp/language-reference/keywords/using-statement
            using ( var port = new SerialPort(s_portName, s_baudrate, s_parity, s_dataBits, s_stopBits) ) {
                // お使いの通信ポートによっては追加の設定が必要になります
                // 詳細: https://learn.microsoft.com/ja-jp/dotnet/api/system.io.ports.serialport?view=netframework-4.8
                port.NewLine = s_delimiter;
                port.ReadTimeout = 1000;
                port.WriteTimeout = 1000;

                // シリアル通信ポートを開きます。
                port.Open();

                // MGO コマンドを送信します。
                var cmd = "MGO:A+1000";
                port.WriteLine(cmd);
                Console.WriteLine($"通信ポート {s_portName} にコマンド {cmd} を送信しました。");

                // この部分は移動がある程度進むのを待つためのものなのでコントローラとの通信とは関係ありません。
                Task.Delay(100).Wait();

                // Q コマンドを送信し、返答を受信します。
                cmd = "Q:A0";
                port.WriteLine(cmd);
                Console.WriteLine($"通信ポート {s_portName} にコマンド {cmd} を送信しました。");
                var readed = port.ReadLine();
                Console.WriteLine($"通信ポート {s_portName} からデータ {readed} を受信しました。");


                // この部分は受信した文字の処理なのでコントローラとの通信には関係ありません。
                char stat;
                if ( Int32.TryParse(readed.Substring(0, 9), out var pos) is false ) {
                    // 位置を整数に変換できない場合、意図的に QT-A / QT-B で表現できない Int32.MinValue を入れています。
                    pos = Int32.MinValue;
                }
                stat = readed.Last();
                Console.WriteLine($"A 軸: {pos} + {stat}");
                Console.WriteLine("何かキーを押してください。");
                _ = Console.ReadKey();
            }
        }
    }
}
